@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright 2008 Øyvind Kolås <pippin@gimp.org>
+ * Copyright 2008,2011,2012,2014,2017 Øyvind Kolås <pippin@gimp.org>
  *           2013 Daniel Sabo
  */
 
@@ -187,13 +187,8 @@ release_tile (GeglBufferIterator *iter,
     {
       if (sub->access_mode & GEGL_ACCESS_WRITE)
         {
-          GeglRectangle roi = {sub->real_roi.x << sub->level,
-                               sub->real_roi.y << sub->level,
-                               sub->real_roi.width << sub->level,
-                               sub->real_roi.height << sub->level};
-
           gegl_buffer_set_unlocked_no_notify (sub->buffer,
-                                              &roi,
+                                              &sub->real_roi,
                                               sub->level,
                                               sub->format,
                                               sub->real_data,
