@@ -26,7 +26,7 @@
 
 #include "gegl.h"
 #include "gegl-types-internal.h"
-#include "graph/gegl-node.h"
+#include "graph/gegl-node-private.h"
 #include "graph/gegl-pad.h"
 #include "graph/gegl-connection.h"
 #include "graph/gegl-visitable.h"
@@ -79,7 +79,7 @@ gegl_dot_util_add_node (GString  *string,
   if (1)
     {
       guint        n_properties;
-      GParamSpec **properties = gegl_list_properties (gegl_node_get_operation (node), &n_properties);
+      GParamSpec **properties = gegl_operation_list_properties (gegl_node_get_operation (node), &n_properties);
       guint        i;
       for (i = 0; i < n_properties; i++)
         {
@@ -188,7 +188,7 @@ gegl_dot_add_graph (GString     *string,
 {
   GeglNode *graph = node;
 
-  g_string_append_printf (string, "subgraph cluster_%s%p { graph [ label=\"%s %p\" fontsize=\"10\" ranksep=\"0.3\" nodesep=\"0.3\"]; node [ fontsize=\"10\" ];\n", label, node, label, node);
+  g_string_append_printf (string, "subgraph cluster_%p { graph [ label=\"%s %p\" fontsize=\"10\" ranksep=\"0.3\" nodesep=\"0.3\"]; node [ fontsize=\"10\" ];\n", node, label, node);
 
   {
     GSList *nodes = gegl_node_get_children (graph);
@@ -350,4 +350,5 @@ gegl_dot_node_to_png (GeglNode    *node,
   dot_cmd = g_strdup_printf ("dot -o %s -Tpng %s", png_path, dot_filename);
   if (system (dot_cmd) == -1)
     g_warning ("Error executing GraphViz dot program");
+  g_free (dot_cmd);
 }
