@@ -29,7 +29,15 @@
 #define GEGL_HEADER(x) _GEGL_HEADER(x)
 
 #define GEGL_OP_C_FILE GEGL_HEADER(GEGL_OP_C_SOURCE)
+#ifndef GEGL_OP_NO_SOURCE
 #define GEGL_OP_C_FILE_SOURCE GEGL_HEADER(GEGL_OP_C_SOURCE.h)
+#endif
+#endif
+
+#ifndef GEGL_OP_NAME
+#define GEGL_OP_NAME op
+#error "define GEGL_OP_NAME"
+
 #endif
 
 #ifndef GEGL_OP_C_FILE
@@ -39,8 +47,6 @@
 #error "GEGL_OP_C_FILE not defined"
 #endif
 
-#define gegl_chant_class_init   gegl_chant_class_init_SHOULD_BE_gegl_op_class_init
-
 /* XXX: */
 #ifdef GEGL_CHANT_TYPE_POINT_RENDER
 #error "GEGL_CHANT_TYPE_POINT_RENDER should be replaced with GEGL_OP_POINT_RENDER"
@@ -54,32 +60,39 @@
 
 G_BEGIN_DECLS
 
-static GType gegl_op_get_type (void) G_GNUC_UNUSED;
 typedef struct _GeglProperties  GeglProperties;
 typedef struct _GeglOp   GeglOp;
 
-static void gegl_op_register_type       (GTypeModule *module);
+
 static void gegl_op_init_properties     (GeglOp   *self);
 static void gegl_op_class_intern_init   (gpointer     klass);
 static gpointer gegl_op_parent_class = NULL;
 
+#define define_register_type_(a) \
+void gegl_op_##a##_register_type     (GTypeModule *module);\
+GType gegl_op_##a##_get_type (void);
+#define define_register_type(a) define_register_type_(a)
+define_register_type(GEGL_OP_NAME)
+#undef define_register_type
+#undef define_register_type_
+
 #define GEGL_DEFINE_DYNAMIC_OPERATION_EXTENDED(C_FILE, TypeName, type_name, TYPE_PARENT, flags, CODE) \
 static void     type_name##_init              (TypeName        *self);  \
-static void     type_name##_class_init        (TypeName##Class *klass); \
+static void     gegl_op_class_init            (TypeName##Class *klass); \
 static void     type_name##_class_finalize    (TypeName##Class *klass); \
 static GType    type_name##_type_id = 0;                                \
 static void     type_name##_class_chant_intern_init (gpointer klass)    \
   {                                                                     \
     gegl_op_parent_class = g_type_class_peek_parent (klass);            \
     gegl_op_class_intern_init (klass);                                  \
-    type_name##_class_init ((TypeName##Class*) klass);                  \
+    gegl_op_class_init ((TypeName##Class*) klass);                      \
   }                                                                     \
-static GType                                                            \
+GType                                                            \
 type_name##_get_type (void)                                             \
   {                                                                     \
     return type_name##_type_id;                                         \
   }                                                                     \
-static void                                                             \
+void                                                             \
 type_name##_register_type (GTypeModule *type_module)                    \
   {                                                                     \
     gchar  tempname[256];                                               \
@@ -110,8 +123,9 @@ type_name##_register_type (GTypeModule *type_module)                    \
     { CODE ; }                                                          \
   }
 
-#define GEGL_DEFINE_DYNAMIC_OPERATION_(T_P, C_FILE) GEGL_DEFINE_DYNAMIC_OPERATION_EXTENDED(C_FILE, GeglOp, gegl_op, T_P, 0, {})
-#define GEGL_DEFINE_DYNAMIC_OPERATION(T_P) GEGL_DEFINE_DYNAMIC_OPERATION_(T_P, GEGL_OP_C_FILE)
+#define GEGL_DEFINE_DYNAMIC_OPERATION_(T_P, C_FILE, opname) GEGL_DEFINE_DYNAMIC_OPERATION_EXTENDED(C_FILE, GeglOp, gegl_op_##opname, T_P, 0, {})
+#define GEGL_DEFINE_DYNAMIC_OPERATION__(a,b,c) GEGL_DEFINE_DYNAMIC_OPERATION_(a,b,c)
+#define GEGL_DEFINE_DYNAMIC_OPERATION(T_P) GEGL_DEFINE_DYNAMIC_OPERATION__(T_P, GEGL_OP_C_FILE, GEGL_OP_NAME)
 
 
 #define GEGL_PROPERTIES(op) ((GeglProperties *) (((GeglOp*)(op))->properties))
@@ -180,6 +194,7 @@ type_name##_register_type (GTypeModule *type_module)                    \
 #define GEGL_OP_PARENT GEGL_TYPE_OPERATION_META
 #endif
 
+
 #ifdef GEGL_OP_Parent
 struct _GeglOp
 {
@@ -205,16 +220,14 @@ GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_OP_PARENT)
  * code or your own implementation of it
  */
 #ifndef GEGL_OP_CUSTOM
-static void
-gegl_op_init (GeglOp *self)
-{
-  gegl_op_init_properties (self);
-}
 
-static void
-gegl_op_class_finalize (GeglOpClass *self)
-{
-}
+#define define_init_(a) \
+  static void gegl_op_##a##_init (GeglOp *self) { gegl_op_init_properties (self); }\
+  static void gegl_op_##a##_class_finalize (GeglOpClass *self) { }
+#define define_init(a) define_init_(a)
+define_init(GEGL_OP_NAME)
+#undef define_init
+#undef define_init_
 
 static const GeglModuleInfo modinfo =
 {
@@ -225,6 +238,7 @@ static const GeglModuleInfo modinfo =
 gboolean                gegl_module_register (GTypeModule *module);
 const GeglModuleInfo  * gegl_module_query    (GTypeModule *module);
 
+#ifndef GEGL_OP_BUNDLE
 G_MODULE_EXPORT const GeglModuleInfo *
 gegl_module_query (GTypeModule *module)
 {
@@ -234,11 +248,14 @@ gegl_module_query (GTypeModule *module)
 G_MODULE_EXPORT gboolean
 gegl_module_register (GTypeModule *module)
 {
-  gegl_op_register_type (module);
-
+#define do_reg_(a) gegl_op_##a##_register_type (module)
+#define do_reg(b) do_reg_(b)
+  do_reg(GEGL_OP_NAME);
+#undef do_reg
   return TRUE;
 }
 #endif
+#endif
 
 
 #define description(blurb)
@@ -268,6 +285,7 @@ gegl_module_register (GTypeModule *module)
 #define property_curve(name, label, def_val)      ITEM2(name,label,def_val,object)
 #define property_path(name, label, def_val)       ITEM2(name,label,def_val,object)
 #define property_color(name, label, def_val)      ITEM2(name,label,def_val,object)
+#define property_audio_fragment(name, label, def_val)      ITEM2(name,label,NULL,object)
 
 #define enum_start(enum_name)   typedef enum {
 #define enum_value(value, nick, name)    value ,
@@ -331,6 +349,7 @@ static GType enum_name ## _get_type (void)               \
 #undef property_seed
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
 #undef enum_start
 #undef enum_value
@@ -353,6 +372,7 @@ struct _GeglProperties
 #define property_object(name, label, def_val)          GObject    *name;
 #define property_curve(name, label, def_val)           GeglCurve  *name;
 #define property_color(name, label, def_val)           GeglColor  *name;
+#define property_audio_fragment(name, label, def_val)  GeglAudioFragment  *name;
 #define property_path(name, label, def_val)            GeglPath   *name; gulong path_changed_handler;
 #define property_pointer(name, label, def_val)         gpointer    name;
 #define property_format(name, label, def_val)          gpointer    name;
@@ -375,6 +395,7 @@ struct _GeglProperties
 #undef property_seed
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
 };
 
@@ -397,6 +418,7 @@ enum
 #define property_object(name, label, def_val)     ITEM2(name,label,def_val,object)
 #define property_curve(name, label, def_val)      ITEM2(name,label,def_val,object)
 #define property_color(name, label, def_val)      ITEM2(name,label,def_val,object)
+#define property_audio_fragment(name, label, def_val) ITEM2(name,label,def_val,object)
 #define property_path(name, label, def_val)       ITEM2(name,label,def_val,object)
 #define property_pointer(name, label, def_val)    ITEM(name,label,def_val,pointer)
 #define property_format(name, label, def_val)     ITEM(name,label,def_val,pointer)
@@ -443,6 +465,7 @@ get_property (GObject      *gobject,
 #undef property_seed
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -511,6 +534,12 @@ set_property (GObject      *gobject,
          g_object_unref (properties->name);                           \
       properties->name = g_value_dup_object (value);                  \
       break;
+#define property_audio_fragment(name, label, def_val)                 \
+    case PROP_##name:                                                 \
+      if (properties->name != NULL)                                   \
+         g_object_unref (properties->name);                           \
+      properties->name = g_value_dup_object (value);                  \
+      break;
 #define property_path(name, label, def_val)                           \
     case PROP_##name:                                                 \
       if (properties->name != NULL)                                   \
@@ -562,6 +591,7 @@ set_property (GObject      *gobject,
 #undef property_pointer
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
 #undef property_format
 #undef property_enum
@@ -618,7 +648,13 @@ static void gegl_op_destroy_notify (gpointer data)
       g_object_unref (properties->name);            \
       properties->name = NULL;                      \
     }
-#define property_path(name, label, def_val)       \
+#define property_audio_fragment(name, label, def_val) \
+  if (properties->name)                             \
+    {                                               \
+      g_object_unref (properties->name);            \
+      properties->name = NULL;                      \
+    }
+#define property_path(name, label, def_val)         \
   if (properties->name)                             \
     {                                               \
       g_object_unref (properties->name);            \
@@ -644,6 +680,7 @@ static void gegl_op_destroy_notify (gpointer data)
 #undef property_seed
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
 
   g_slice_free (GeglProperties, properties);
@@ -673,6 +710,9 @@ gegl_op_constructor (GType                  type,
 #define property_pointer(name, label, def_val)
 #define property_format(name, label, def_val)
 #define property_curve(name, label, def_val)
+#define property_audio_fragment(name, label, def_val)\
+    if (properties->name == NULL)                   \
+    {properties->name = gegl_audio_fragment_new(48000, 2, 0, 8192);}
 #define property_color(name, label, def_val)\
     if (properties->name == NULL)                   \
     {properties->name = gegl_color_new(def_val?def_val:"black");}
@@ -697,6 +737,7 @@ gegl_op_constructor (GType                  type,
 #undef property_seed
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_path
 
   g_object_set_data_full (obj, "chant-data", obj, gegl_op_destroy_notify);
@@ -940,6 +981,12 @@ gegl_op_class_intern_init (gpointer klass)
        gegl_param_spec_color_from_string (#name, label, NULL, def_val, flags);\
      current_prop = PROP_##name ;
 
+#define property_audio_fragment(name, label, def_val) \
+    REGISTER_IF_ANY  \
+  }{ GParamSpec *pspec = \
+       gegl_param_spec_audio_fragment (#name, label, NULL, flags);\
+     current_prop = PROP_##name ;
+
 #define property_path(name, label, def_val) \
     REGISTER_IF_ANY  \
   }{ GParamSpec *pspec = \
@@ -989,6 +1036,7 @@ gegl_op_class_intern_init (gpointer klass)
 #undef property_path
 #undef property_curve
 #undef property_color
+#undef property_audio_fragment
 #undef property_object
 #undef property_format
 
