@@ -39,21 +39,49 @@ typedef enum
   GEGL_BLIT_DIRTY    = 1 << 1
 } GeglBlitFlags;
 
-typedef struct _GeglConfig GeglConfig;
-typedef struct _GeglCurve  GeglCurve;
-typedef struct _GeglPath   GeglPath;
-typedef struct _GeglColor  GeglColor;
 
-typedef struct _GeglRectangle
+typedef struct _GeglConfig GeglConfig;
+GType gegl_config_get_type (void) G_GNUC_CONST;
+#define GEGL_TYPE_CONFIG            (gegl_config_get_type ())
+#define GEGL_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_CONFIG, GeglConfig))
+#define GEGL_IS_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_CONFIG))
+
+typedef struct _GeglSampler       GeglSampler;
+typedef struct _GeglCurve         GeglCurve;
+typedef struct _GeglPath          GeglPath;
+typedef struct _GeglColor         GeglColor;
+typedef struct _GeglAudioFragment GeglAudioFragment;
+
+typedef struct _GeglRectangle GeglRectangle;
+
+struct _GeglRectangle
 {
   gint x;
   gint y;
   gint width;
   gint height;
-} GeglRectangle;
+};
 GType gegl_rectangle_get_type (void) G_GNUC_CONST;
 #define GEGL_TYPE_RECTANGLE   (gegl_rectangle_get_type())
 
+#define  GEGL_RECTANGLE(x,y,w,h) (&((GeglRectangle){(x), (y),   (w), (h)}))
+
+typedef struct _GeglTileBackend GeglTileBackend;
+
+typedef struct _GeglOperationContext GeglOperationContext;
+
+typedef struct _GeglBuffer  GeglBuffer;
+GType gegl_buffer_get_type  (void) G_GNUC_CONST;
+#define GEGL_TYPE_BUFFER    (gegl_buffer_get_type ())
+#define GEGL_BUFFER(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
+#define GEGL_IS_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_BUFFER))
+
+typedef struct _GeglOperation  GeglOperation;
+GType gegl_operation_get_type  (void) G_GNUC_CONST;
+#define GEGL_TYPE_OPERATION    (gegl_operation_get_type ())
+#define GEGL_OPERATION(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_OPERATION, GeglOperation))
+#define GEGL_IS_OPERATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_OPERATION))
+
 typedef struct _GeglNode  GeglNode;
 GType gegl_node_get_type  (void) G_GNUC_CONST;
 #define GEGL_TYPE_NODE    (gegl_node_get_type())
@@ -66,10 +94,9 @@ GType gegl_processor_get_type  (void) G_GNUC_CONST;
 #define GEGL_PROCESSOR(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_PROCESSOR, GeglProcessor))
 #define GEGL_IS_PROCESSOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_PROCESSOR))
 
-
-typedef void         (*GeglDestroyNotify)      (gpointer pixel_data,
-                                                gpointer user_data);
-
+typedef struct _GeglRandom  GeglRandom;
+GType gegl_random_get_type  (void) G_GNUC_CONST;
+#define GEGL_TYPE_RANDOM    (gegl_random_get_type())
 
 G_END_DECLS
 
