@@ -220,9 +220,7 @@ gegl_operation_filter_process (GeglOperation        *operation,
     success = klass->process (operation, input, output, result, level);
   }
 
-  if (input != NULL)
-    g_object_unref (input);
-
+  g_clear_object (&input);
   return success;
 }
 
