@@ -49,6 +49,10 @@ static const char* op_c_source =
 "                                                                              \n"
 "#define GEGL_OP_POINT_FILTER                                                  \n"
 "                                                                              \n"
+"/* The C prefix used for some generated functions                             \n"
+" */                                                                           \n"
+"#define GEGL_OP_NAME     brightness_contrast                                  \n"
+"                                                                              \n"
 "/* We specify the file we're in, this is needed to make the code              \n"
 " * generation for the properties work.                                        \n"
 " */                                                                           \n"
