@@ -31,7 +31,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
-"#define GEGL_OP_AREA_FILTER                                                   \n"
+"#define GEGL_OP_FILTER                                                        \n"
 "#define GEGL_OP_NAME     cartoon                                              \n"
 "#define GEGL_OP_C_SOURCE cartoon.c                                            \n"
 "                                                                              \n"
@@ -102,51 +102,48 @@ static const char* op_c_source =
 "}                                                                             \n"
 "                                                                              \n"
 "static gdouble                                                                \n"
-"compute_ramp (GeglSampler         *sampler1,                                  \n"
-"              GeglSampler         *sampler2,                                  \n"
-"              const GeglRectangle *roi,                                       \n"
-"              gdouble              pct_black)                                 \n"
+"compute_ramp (GeglBuffer  *dest1,                                             \n"
+"              GeglBuffer  *dest2,                                             \n"
+"              gdouble      pct_black)                                         \n"
 "{                                                                             \n"
+"  GeglBufferIterator *iter;                                                   \n"
 "  gint    hist[100];                                                          \n"
-"  gdouble diff;                                                               \n"
 "  gint    count;                                                              \n"
-"  gfloat pixel1, pixel2;                                                      \n"
-"  gint x;                                                                     \n"
-"  gint y;                                                                     \n"
 "  gint i;                                                                     \n"
 "  gint sum;                                                                   \n"
 "                                                                              \n"
 "  memset (hist, 0, sizeof (int) * 100);                                       \n"
 "  count = 0;                                                                  \n"
 "                                                                              \n"
-"  for (y = roi->y; y < roi->y + roi->height; ++y)                             \n"
-"    for (x = roi->x; x < roi->x + roi->width; ++x)                            \n"
-"      {                                                                       \n"
-"        gegl_sampler_get (sampler1,                                           \n"
-"                          x,                                                  \n"
-"                          y,                                                  \n"
-"                          NULL,                                               \n"
-"                          &pixel1,                                            \n"
-"                          GEGL_ABYSS_NONE);                                   \n"
+"  iter = gegl_buffer_iterator_new (dest1, NULL, 0, babl_format (\"Y' float\"),\n"
+"                                   GEGL_ACCESS_READ, GEGL_ABYSS_NONE);        \n"
 "                                                                              \n"
-"        gegl_sampler_get (sampler2,                                           \n"
-"                          x,                                                  \n"
-"                          y,                                                  \n"
-"                          NULL,                                               \n"
-"                          &pixel2,                                            \n"
-"                          GEGL_ABYSS_NONE);                                   \n"
+"  gegl_buffer_iterator_add (iter, dest2, NULL, 0, babl_format (\"Y' float\"), \n"
+"                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);               \n"
 "                                                                              \n"
-"        if (pixel2 != 0)                                                      \n"
-"          {                                                                   \n"
-"            diff = (gdouble) pixel1 / (gdouble) pixel2;                       \n"
+"  while (gegl_buffer_iterator_next (iter))                                    \n"
+"    {                                                                         \n"
+"      gfloat *pixel1   = iter->data[0];                                       \n"
+"      gfloat *pixel2   = iter->data[1];                                       \n"
+"      glong   n_pixels = iter->length;                                        \n"
 "                                                                              \n"
-"            if (diff < 1.0 && diff >= 0.0)                                    \n"
-"              {                                                               \n"
-"                hist[(int) (diff * 100)] += 1;                                \n"
-"                count += 1;                                                   \n"
-"              }                                                               \n"
-"          }                                                                   \n"
+"      while (n_pixels--)                                                      \n"
+"        {                                                                     \n"
+"          if (*pixel2 != 0)                                                   \n"
+"            {                                                                 \n"
+"              gdouble diff = (gdouble) *pixel1 / (gdouble) *pixel2;           \n"
+"                                                                              \n"
+"              if (diff < 1.0 && diff >= 0.0)                                  \n"
+"                {                                                             \n"
+"                  hist[(int) (diff * 100)] += 1;                              \n"
+"                  count += 1;                                                 \n"
+"                }                                                             \n"
+"            }                                                                 \n"
+"                                                                              \n"
+"          pixel1++;                                                           \n"
+"          pixel2++;                                                           \n"
 "      }                                                                       \n"
+"  }                                                                           \n"
 "                                                                              \n"
 "  if (pct_black == 0.0 || count == 0)                                         \n"
 "    return 1.0;                                                               \n"
@@ -160,16 +157,13 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "                                                                              \n"
 "  return 0.0;                                                                 \n"
-"                                                                              \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
 "prepare (GeglOperation *operation)                                            \n"
 "{                                                                             \n"
-"  gegl_operation_set_format (operation, \"input\",                            \n"
-"                             babl_format (\"Y'CbCrA float\"));                \n"
-"  gegl_operation_set_format (operation, \"output\",                           \n"
-"                             babl_format (\"Y'CbCrA float\"));                \n"
+"  gegl_operation_set_format (operation, \"input\",  babl_format (\"RGBA float\"));\n"
+"  gegl_operation_set_format (operation, \"output\", babl_format (\"RGBA float\"));\n"
 "}                                                                             \n"
 "                                                                              \n"
 "static GeglRectangle                                                          \n"
@@ -221,35 +215,28 @@ static const char* op_c_source =
 "  GeglBufferIterator *iter;                                                   \n"
 "  GeglBuffer         *dest1;                                                  \n"
 "  GeglBuffer         *dest2;                                                  \n"
-"  GeglSampler        *sampler1;                                               \n"
-"  GeglSampler        *sampler2;                                               \n"
 "  gdouble             ramp;                                                   \n"
-"  gint                x;                                                      \n"
-"  gint                y;                                                      \n"
-"  gfloat              tot_pixels = result->width * result->height;            \n"
-"  gfloat              pixels = 0;                                             \n"
+"  gdouble             progress = 0.0;                                         \n"
+"  gdouble             pixels_count = result->width * result->height;          \n"
 "                                                                              \n"
 "  grey_blur_buffer (input, o->mask_radius, &dest1, &dest2);                   \n"
 "                                                                              \n"
-"  sampler1 = gegl_buffer_sampler_new_at_level (dest1,                         \n"
-"                                               babl_format (\"Y' float\"),    \n"
-"                                               GEGL_SAMPLER_LINEAR,           \n"
-"                                               level);                        \n"
-"                                                                              \n"
-"  sampler2 = gegl_buffer_sampler_new_at_level (dest2,                         \n"
-"                                               babl_format (\"Y' float\"),    \n"
-"                                               GEGL_SAMPLER_LINEAR,           \n"
-"                                               level);                        \n"
-"                                                                              \n"
-"  ramp = compute_ramp (sampler1, sampler2, result, o->pct_black);             \n"
+"  ramp = compute_ramp (dest1, dest2, o->pct_black);                           \n"
 "                                                                              \n"
 "  iter = gegl_buffer_iterator_new (output, result, 0,                         \n"
 "                                   babl_format (\"Y'CbCrA float\"),           \n"
 "                                   GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);       \n"
+"                                                                              \n"
 "  gegl_buffer_iterator_add (iter, input, result, 0,                           \n"
 "                            babl_format (\"Y'CbCrA float\"),                  \n"
 "                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);               \n"
 "                                                                              \n"
+"  gegl_buffer_iterator_add (iter, dest1, NULL, 0, babl_format (\"Y' float\"), \n"
+"                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);               \n"
+"                                                                              \n"
+"  gegl_buffer_iterator_add (iter, dest2, NULL, 0, babl_format (\"Y' float\"), \n"
+"                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);               \n"
+"                                                                              \n"
 "                                                                              \n"
 "  gegl_operation_progress (operation, 0.0, \"\");                             \n"
 "                                                                              \n"
@@ -257,57 +244,47 @@ static const char* op_c_source =
 "    {                                                                         \n"
 "      gfloat *out_pixel = iter->data[0];                                      \n"
 "      gfloat *in_pixel  = iter->data[1];                                      \n"
+"      gfloat *grey1     = iter->data[2];                                      \n"
+"      gfloat *grey2     = iter->data[3];                                      \n"
+"      glong   n_pixels  = iter->length;                                       \n"
 "                                                                              \n"
-"      for (y = iter->roi[0].y; y < iter->roi[0].y + iter->roi[0].height; ++y) \n"
-"      {                                                                       \n"
-"        for (x = iter->roi[0].x; x < iter->roi[0].x + iter->roi[0].width; ++x)\n"
-"          {                                                                   \n"
-"            gfloat  pixel1;                                                   \n"
-"            gfloat  pixel2;                                                   \n"
-"            gdouble mult = 0.0;                                               \n"
-"            gdouble diff;                                                     \n"
+"      progress += n_pixels / pixels_count;                                    \n"
 "                                                                              \n"
-"            gegl_sampler_get (sampler1, x, y,                                 \n"
-"                              NULL, &pixel1,                                  \n"
-"                              GEGL_ABYSS_NONE);                               \n"
+"      while (n_pixels--)                                                      \n"
+"        {                                                                     \n"
+"          gdouble mult = 0.0;                                                 \n"
 "                                                                              \n"
-"            gegl_sampler_get (sampler2, x, y,                                 \n"
-"                              NULL, &pixel2,                                  \n"
-"                              GEGL_ABYSS_NONE);                               \n"
+"          if (*grey2 != 0)                                                    \n"
+"            {                                                                 \n"
+"              gdouble diff = (gdouble) *grey1 / (gdouble) *grey2;             \n"
 "                                                                              \n"
-"            if (pixel2 != 0)                                                  \n"
-"              {                                                               \n"
-"                diff = (gdouble) pixel1 / (gdouble) pixel2;                   \n"
-"                if (diff < THRESHOLD)                                         \n"
-"                  {                                                           \n"
-"                    if (GEGL_FLOAT_EQUAL (ramp, 0.0))                         \n"
-"                      mult = 0.0;                                             \n"
-"                    else                                                      \n"
-"                      mult = (ramp - MIN (ramp, (THRESHOLD - diff))) / ramp;  \n"
-"                  }                                                           \n"
-"                else                                                          \n"
-"                  mult = 1.0;                                                 \n"
-"              }                                                               \n"
+"              if (diff < THRESHOLD)                                           \n"
+"                {                                                             \n"
+"                  if (GEGL_FLOAT_EQUAL (ramp, 0.0))                           \n"
+"                    mult = 0.0;                                               \n"
+"                  else                                                        \n"
+"                    mult = (ramp - MIN (ramp, (THRESHOLD - diff))) / ramp;    \n"
+"                }                                                             \n"
+"              else                                                            \n"
+"                mult = 1.0;                                                   \n"
+"            }                                                                 \n"
 "                                                                              \n"
-"            out_pixel[0] = CLAMP (pixel1 * mult, 0.0, 1.0);                   \n"
-"            out_pixel[1] = in_pixel[1];                                       \n"
-"            out_pixel[2] = in_pixel[2];                                       \n"
-"            out_pixel[3] = in_pixel[3];                                       \n"
+"          out_pixel[0] = CLAMP (*grey1 * mult, 0.0, 1.0);                     \n"
+"          out_pixel[1] = in_pixel[1];                                         \n"
+"          out_pixel[2] = in_pixel[2];                                         \n"
+"          out_pixel[3] = in_pixel[3];                                         \n"
 "                                                                              \n"
-"            out_pixel += 4;                                                   \n"
-"            in_pixel  += 4;                                                   \n"
+"          out_pixel += 4;                                                     \n"
+"          in_pixel  += 4;                                                     \n"
+"          grey1++;                                                            \n"
+"          grey2++;                                                            \n"
+"        }                                                                     \n"
 "                                                                              \n"
-"          }                                                                   \n"
-"        pixels += iter->roi[0].width;                                         \n"
-"        gegl_operation_progress (operation, pixels / tot_pixels, \"\");       \n"
-"      }                                                                       \n"
+"        gegl_operation_progress (operation, progress, \"\");                  \n"
 "    }                                                                         \n"
 "                                                                              \n"
 "  gegl_operation_progress (operation, 1.0, \"\");                             \n"
 "                                                                              \n"
-"  g_object_unref (sampler1);                                                  \n"
-"  g_object_unref (sampler2);                                                  \n"
-"                                                                              \n"
 "  g_object_unref (dest1);                                                     \n"
 "  g_object_unref (dest2);                                                     \n"
 "                                                                              \n"
@@ -336,7 +313,10 @@ static const char* op_c_source =
 "    \"name\",        \"gegl:cartoon\",                                        \n"
 "    \"title\",       _(\"Cartoon\"),                                          \n"
 "    \"license\",     \"GPL3+\",                                               \n"
-"    \"description\", _(\"Simulates a cartoon, its result is similar to a black felt pen drawing subsequently shaded with color. This is achieved by enhancing edges and darkening areas that are already distinctly darker than their neighborhood\"),\n"
+"    \"description\", _(\"Simulates a cartoon, its result is similar to a black\"\n"
+"                     \" felt pen drawing subsequently shaded with color. This\"\n"
+"                     \" is achieved by enhancing edges and darkening areas that\"\n"
+"                     \" are already distinctly darker than their neighborhood\"),\n"
 "    NULL);                                                                    \n"
 "}                                                                             \n"
 "                                                                              \n"
