@@ -173,9 +173,10 @@ static const char* op_c_source =
 "}                                                                             \n"
 "                                                                              \n"
 "static gboolean                                                               \n"
-"normalize_o (GeglProperties  *o,                                              \n"
-"             gfloat           matrix[MAX_MATRIX_SIZE][MAX_MATRIX_SIZE],       \n"
-"             gint             matrix_size)                                    \n"
+"normalize_div_off (gfloat  matrix[MAX_MATRIX_SIZE][MAX_MATRIX_SIZE],          \n"
+"                   gint    matrix_size,                                       \n"
+"                   gfloat *divisor,                                           \n"
+"                   gfloat *offset)                                            \n"
 "{                                                                             \n"
 "  gint      x, y;                                                             \n"
 "  gboolean  valid = FALSE;                                                    \n"
@@ -191,18 +192,18 @@ static const char* op_c_source =
 "                                                                              \n"
 "  if (sum > 0)                                                                \n"
 "    {                                                                         \n"
-"      o->offset  = 0.0;                                                       \n"
-"      o->divisor = sum;                                                       \n"
+"      *offset  = 0.0;                                                         \n"
+"      *divisor = sum;                                                         \n"
 "    }                                                                         \n"
 "  else if (sum < 0)                                                           \n"
 "    {                                                                         \n"
-"      o->offset  = 1.0;                                                       \n"
-"      o->divisor = -sum;                                                      \n"
+"      *offset  = 1.0;                                                         \n"
+"      *divisor = -sum;                                                        \n"
 "    }                                                                         \n"
 "  else                                                                        \n"
 "    {                                                                         \n"
-"      o->offset  = 0.5;                                                       \n"
-"      o->divisor = 1;                                                         \n"
+"      *offset  = 0.5;                                                         \n"
+"      *divisor = 1;                                                           \n"
 "    }                                                                         \n"
 "                                                                              \n"
 "  return valid;                                                               \n"
@@ -221,7 +222,8 @@ static const char* op_c_source =
 "                gint                  xx,                                     \n"
 "                gint                  yy,                                     \n"
 "                gfloat                matrixsum,                              \n"
-"                gfloat                inv_divisor)                            \n"
+"                gfloat                inv_divisor,                            \n"
+"                gfloat                offset)                                 \n"
 "{                                                                             \n"
 "  gint   i;                                                                   \n"
 "  gint   s_stride = (extended->width - matrix_size) * 4;                      \n"
@@ -246,7 +248,7 @@ static const char* op_c_source =
 "            s_offset += s_stride;                                             \n"
 "          }                                                                   \n"
 "          sum = sum * inv_divisor;                                            \n"
-"          sum += o->offset;                                                   \n"
+"          sum += offset;                                                      \n"
 "          dst_buf[d_offset + i] = sum;                                        \n"
 "        }                                                                     \n"
 "      else                                                                    \n"
@@ -269,7 +271,8 @@ static const char* op_c_source =
 "                gint                  xx,                                     \n"
 "                gint                  yy,                                     \n"
 "                gfloat                matrixsum,                              \n"
-"                gfloat                inv_divisor)                            \n"
+"                gfloat                inv_divisor,                            \n"
+"                gfloat                offset)                                 \n"
 "{                                                                             \n"
 "  gint   i;                                                                   \n"
 "  gint   s_stride = (extended->width - matrix_size) * 4;                      \n"
@@ -288,7 +291,7 @@ static const char* op_c_source =
 "        s_offset += s_stride;                                                 \n"
 "      }                                                                       \n"
 "      sum = sum * inv_divisor;                                                \n"
-"      sum += o->offset;                                                       \n"
+"      sum += offset;                                                          \n"
 "      dst_buf[d_offset + i] = sum;                                            \n"
 "    }                                                                         \n"
 "}                                                                             \n"
@@ -306,7 +309,8 @@ static const char* op_c_source =
 "                             gint                  xx,                        \n"
 "                             gint                  yy,                        \n"
 "                             gfloat                matrixsum,                 \n"
-"                             gfloat                inv_divisor)               \n"
+"                             gfloat                inv_divisor,               \n"
+"                             gfloat                offset)                    \n"
 "{                                                                             \n"
 "  gint   i;                                                                   \n"
 "  gint   s_stride = (extended->width - matrix_size) * 4;                      \n"
@@ -325,7 +329,7 @@ static const char* op_c_source =
 "          }                                                                   \n"
 "        s_offset += s_stride;                                                 \n"
 "      }                                                                       \n"
-"      sum = sum * inv_divisor + o->offset;                                    \n"
+"      sum = sum * inv_divisor + offset;                                       \n"
 "      dst_buf[d_offset + i] = sum;                                            \n"
 "    }                                                                         \n"
 "    {                                                                         \n"
@@ -349,10 +353,10 @@ static const char* op_c_source =
 "      if (alphasum > 0.0)                                                     \n"
 "      {                                                                       \n"
 "        sum = sum * inv_divisor;                                              \n"
-"        sum = sum * matrixsum / alphasum + o->offset;                         \n"
+"        sum = sum * matrixsum / alphasum + offset;                            \n"
 "      }                                                                       \n"
 "      else                                                                    \n"
-"        sum = o->offset;                                                      \n"
+"        sum = offset;                                                         \n"
 "      dst_buf[d_offset + i] = sum;                                            \n"
 "    }                                                                         \n"
 "}                                                                             \n"
@@ -370,7 +374,8 @@ static const char* op_c_source =
 "                             gint                  xx,                        \n"
 "                             gint                  yy,                        \n"
 "                             gfloat                matrixsum,                 \n"
-"                             gfloat                inv_divisor)               \n"
+"                             gfloat                inv_divisor,               \n"
+"                             gfloat                offset)                    \n"
 "{                                                                             \n"
 "  gint   i;                                                                   \n"
 "  gint   s_stride = (extended->width - matrix_size) * 4;                      \n"
@@ -394,7 +399,7 @@ static const char* op_c_source =
 "              }                                                               \n"
 "            s_offset += s_stride;                                             \n"
 "          }                                                                   \n"
-"          sum = sum * inv_divisor + o->offset;                                \n"
+"          sum = sum * inv_divisor + offset;                                   \n"
 "        }                                                                     \n"
 "      else                                                                    \n"
 "        {                                                                     \n"
@@ -427,10 +432,10 @@ static const char* op_c_source =
 "          if (alphasum != 0)                                                  \n"
 "          {                                                                   \n"
 "            sum = sum * inv_divisor;                                          \n"
-"            sum = sum * matrixsum / alphasum + o->offset;                     \n"
+"            sum = sum * matrixsum / alphasum + offset;                        \n"
 "          }                                                                   \n"
 "          else                                                                \n"
-"            sum = o->offset;                                                  \n"
+"            sum = offset;                                                     \n"
 "        }                                                                     \n"
 "      else                                                                    \n"
 "        {                                                                     \n"
@@ -455,6 +460,8 @@ static const char* op_c_source =
 "  gfloat                  *dst_buf;                                           \n"
 "  gfloat                   matrix[MAX_MATRIX_SIZE][MAX_MATRIX_SIZE]={{0,}};   \n"
 "  gfloat                   matrixsum = 0.0;                                   \n"
+"  gfloat                   divisor = o->divisor;                              \n"
+"  gfloat                   offset = o->offset;                                \n"
 "  gfloat                   inv_divisor;                                       \n"
 "  gint                     x, y;                                              \n"
 "  gint                     matrix_size = MAX_MATRIX_SIZE;                     \n"
@@ -465,8 +472,8 @@ static const char* op_c_source =
 "  make_matrix (o, matrix, matrix_size);                                       \n"
 "                                                                              \n"
 "  if (o->normalize)                                                           \n"
-"    normalize_o (o, matrix, matrix_size);                                     \n"
-"  inv_divisor = 1.0 / o->divisor;                                             \n"
+"    normalize_div_off (matrix, matrix_size, &divisor, &offset);               \n"
+"  inv_divisor = 1.0 / divisor;                                                \n"
 "                                                                              \n"
 "  for (x = 0; x < matrix_size; x++)                                           \n"
 "    for (y = 0; y < matrix_size; y++)                                         \n"
@@ -498,7 +505,7 @@ static const char* op_c_source =
 "              for (x = result->x; x < result->width + result->x; x++)         \n"
 "              {                                                               \n"
 "                convolve_pixel_alpha_weight_componentwise (o, src_buf, dst_buf, result, &rect,\n"
-"                                           matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor);\n"
+"                                           matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor, offset);\n"
 "                d_offset += 4;                                                \n"
 "                ss_offset += 4;                                               \n"
 "              }                                                               \n"
@@ -513,7 +520,7 @@ static const char* op_c_source =
 "              for (x = result->x; x < result->width + result->x; x++)         \n"
 "              {                                                               \n"
 "                convolve_pixel_alpha_weight (o, src_buf, dst_buf, result, &rect,\n"
-"                                           matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor);\n"
+"                                           matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor, offset);\n"
 "                d_offset += 4;                                                \n"
 "                ss_offset += 4;                                               \n"
 "              }                                                               \n"
@@ -531,7 +538,7 @@ static const char* op_c_source =
 "              for (x = result->x; x < result->width + result->x; x++)         \n"
 "              {                                                               \n"
 "                convolve_pixel_componentwise (o, src_buf, dst_buf, result, &rect,\n"
-"                                matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor);\n"
+"                                matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor, offset);\n"
 "                d_offset += 4;                                                \n"
 "                ss_offset += 4;                                               \n"
 "              }                                                               \n"
@@ -546,7 +553,7 @@ static const char* op_c_source =
 "              for (x = result->x; x < result->width + result->x; x++)         \n"
 "              {                                                               \n"
 "                convolve_pixel (o, src_buf, dst_buf, result, &rect,           \n"
-"                                matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor);\n"
+"                                matrix, matrix_size, d_offset, ss_offset, x, y, matrixsum, inv_divisor, offset);\n"
 "                d_offset += 4;                                                \n"
 "                ss_offset += 4;                                               \n"
 "              }                                                               \n"
