@@ -32,10 +32,6 @@ static const char* op_c_source =
 "    description (_(\"Relative brightness change in stops\"))                  \n"
 "    ui_range    (-10.0, 10.0)                                                 \n"
 "                                                                              \n"
-"property_double (gamma, _(\"Gamma adjustment\"), 1.0)                         \n"
-"    value_range (0.01, 10)                                                    \n"
-"    ui_range    (0.01, 3.0)                                                   \n"
-"                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
 "#define GEGL_OP_POINT_FILTER                                                  \n"
@@ -75,7 +71,6 @@ static const char* op_c_source =
 "  gfloat      exposure_negated = (gfloat) -o->exposure;                       \n"
 "  gfloat      gain;                                                           \n"
 "  gfloat      white;                                                          \n"
-"  gfloat      gamma = 1.0 / o->gamma;                                         \n"
 "                                                                              \n"
 "  glong       i;                                                              \n"
 "                                                                              \n"
@@ -86,28 +81,16 @@ static const char* op_c_source =
 "  diff = MAX (white - black_level, 0.01);                                     \n"
 "  gain = 1.0f / diff;                                                         \n"
 "                                                                              \n"
-"  if (gamma == 1.0)                                                           \n"
-"    for (i=0; i<n_pixels; i++)                                                \n"
-"      {                                                                       \n"
-"        out_pixel[0] = (in_pixel[0] - black_level) * gain;                    \n"
-"        out_pixel[1] = (in_pixel[1] - black_level) * gain;                    \n"
-"        out_pixel[2] = (in_pixel[2] - black_level) * gain;                    \n"
-"        out_pixel[3] = in_pixel[3];                                           \n"
+"  for (i=0; i<n_pixels; i++)                                                  \n"
+"    {                                                                         \n"
+"      out_pixel[0] = (in_pixel[0] - black_level) * gain;                      \n"
+"      out_pixel[1] = (in_pixel[1] - black_level) * gain;                      \n"
+"      out_pixel[2] = (in_pixel[2] - black_level) * gain;                      \n"
+"      out_pixel[3] = in_pixel[3];                                             \n"
 "                                                                              \n"
-"        out_pixel += 4;                                                       \n"
-"        in_pixel  += 4;                                                       \n"
-"      }                                                                       \n"
-"  else                                                                        \n"
-"    for (i=0; i<n_pixels; i++)                                                \n"
-"      {                                                                       \n"
-"        out_pixel[0] = powf ((in_pixel[0] - black_level) * gain, gamma);      \n"
-"        out_pixel[1] = powf ((in_pixel[1] - black_level) * gain, gamma);      \n"
-"        out_pixel[2] = powf ((in_pixel[2] - black_level) * gain, gamma);      \n"
-"        out_pixel[3] = in_pixel[3];                                           \n"
-"                                                                              \n"
-"        out_pixel += 4;                                                       \n"
-"        in_pixel += 4;                                                        \n"
-"      }                                                                       \n"
+"      out_pixel += 4;                                                         \n"
+"      in_pixel  += 4;                                                         \n"
+"    }                                                                         \n"
 "                                                                              \n"
 "  return TRUE;                                                                \n"
 "}                                                                             \n"
@@ -118,13 +101,12 @@ static const char* op_c_source =
 "\"__kernel void kernel_exposure(__global const float4 *in,          \\n\"     \n"
 "\"                              __global       float4 *out,         \\n\"     \n"
 "\"                              float                  black_level, \\n\"     \n"
-"\"                              float                  gain,        \\n\"     \n"
-"\"                              float                  gamma)       \\n\"     \n"
+"\"                              float                  gain)        \\n\"     \n"
 "\"{                                                                 \\n\"     \n"
 "\"  int gid = get_global_id(0);                                     \\n\"     \n"
 "\"  float4 in_v  = in[gid];                                         \\n\"     \n"
 "\"  float4 out_v;                                                   \\n\"     \n"
-"\"  out_v.xyz = pow(((in_v.xyz - black_level) * gain), 1.0/gamma);  \\n\"     \n"
+"\"  out_v.xyz =  ((in_v.xyz - black_level) * gain)                  \\n\"     \n"
 "\"  out_v.w   =  in_v.w;                                            \\n\"     \n"
 "\"  out[gid]  =  out_v;                                             \\n\"     \n"
 "\"}                                                                 \\n\";    \n"
@@ -151,7 +133,6 @@ static const char* op_c_source =
 "  gfloat      exposure_negated = (gfloat) -o->exposure;                       \n"
 "  gfloat      gain;                                                           \n"
 "  gfloat      white;                                                          \n"
-"  gfloat      gamma = 1.0 / o->gamma;                                         \n"
 "                                                                              \n"
 "  cl_int cl_err = 0;                                                          \n"
 "                                                                              \n"
@@ -170,7 +151,6 @@ static const char* op_c_source =
 "  cl_err |= gegl_clSetKernelArg(cl_data->kernel[0], 1, sizeof(cl_mem),   (void*)&out_tex);\n"
 "  cl_err |= gegl_clSetKernelArg(cl_data->kernel[0], 2, sizeof(cl_float), (void*)&black_level);\n"
 "  cl_err |= gegl_clSetKernelArg(cl_data->kernel[0], 3, sizeof(cl_float), (void*)&gain);\n"
-"  cl_err |= gegl_clSetKernelArg(cl_data->kernel[0], 4, sizeof(cl_float), (void*)&gamma);\n"
 "  if (cl_err != CL_SUCCESS) return cl_err;                                    \n"
 "                                                                              \n"
 "  cl_err = gegl_clEnqueueNDRangeKernel(gegl_cl_get_command_queue (),          \n"
@@ -202,6 +182,7 @@ static const char* op_c_source =
 "    \"title\",       _(\"Exposure\"),                                         \n"
 "    \"categories\",  \"color\",                                               \n"
 "    \"description\", _(\"Changes Exposure of an image, allows stepping HDR and photographs up/down in stops. \"),\n"
+"    \"op-version\",  \"1:0\",                                                 \n"
 "    NULL);                                                                    \n"
 "}                                                                             \n"
 "                                                                              \n"
