@@ -50,6 +50,7 @@ property_double (noise, _("Noise"), 0.0)
 #else
 
 #define GEGL_OP_FILTER
+#define GEGL_OP_NAME     fattal02
 #define GEGL_OP_C_SOURCE fattal02.c
 
 #include "gegl-op.h"
@@ -1031,9 +1032,11 @@ fattal02_float_cmp (const void *_a,
   const gfloat a = *(gfloat *)_a,
                b = *(gfloat *)_b;
 
-  if (a < b) return -1;
-  if (a > b) return  1;
-             return  0;
+  if (a < b)
+    return -1;
+  if (a > b)
+    return  1;
+  return  0;
 }
 
 
