@@ -14,6 +14,7 @@
  *
  * Copyright 2006 Dominik Ernst <dernst@gmx.de>
  * Copyright 2013 Massimo Valentini <mvalentini@src.gnome.org>
+ *           2017 Øyvind Kolås <pippin@gimp.org>
  *
  * Recursive Gauss IIR Filter as described by Young / van Vliet
  * in "Signal Processing 44 (1995) 139 - 151"
@@ -588,7 +589,6 @@ static GeglGblur1dFilter
 filter_disambiguation (GeglGblur1dFilter filter,
                        gfloat            std_dev)
 {
-  return GEGL_GBLUR_1D_IIR;
   if (filter == GEGL_GBLUR_1D_AUTO)
     {
       /* Threshold 1.0 is arbitrary */
@@ -661,7 +661,7 @@ gegl_gblur_1d_get_required_for_output (GeglOperation       *operation,
                                        const GeglRectangle *output_roi)
 {
   GeglRectangle        required_for_output = { 0, };
-  GeglProperties          *o       = GEGL_PROPERTIES (operation);
+  GeglProperties      *o       = GEGL_PROPERTIES (operation);
   GeglGblur1dFilter    filter  = filter_disambiguation (o->filter, o->std_dev);
 
   if (filter == GEGL_GBLUR_1D_IIR)
@@ -732,7 +732,7 @@ gegl_gblur_1d_get_cached_region (GeglOperation       *operation,
                                  const GeglRectangle *output_roi)
 {
   GeglRectangle      cached_region;
-  GeglProperties        *o       = GEGL_PROPERTIES (operation);
+  GeglProperties    *o       = GEGL_PROPERTIES (operation);
   GeglGblur1dFilter  filter  = filter_disambiguation (o->filter, o->std_dev);
 
   cached_region = *output_roi;
@@ -803,8 +803,6 @@ gegl_gblur_1d_process (GeglOperation       *operation,
   GeglAbyssPolicy   abyss_policy = to_gegl_policy (o->abyss_policy);
 
   GeglRectangle rect2;
-  //level = 0; // mipmap rendering doesn't work yet, unsure if it is
-             // the code in the op or in GeglBuffer
   if (level)
   {
     rect2 = *result;
@@ -812,7 +810,7 @@ gegl_gblur_1d_process (GeglOperation       *operation,
     rect2.y      = result->y >> level;
     rect2.width  = result->width >> level;
     rect2.height = result->height >> level;
-   // result = &rect2;
+    result = &rect2;
     std_dev = std_dev * (1.0/(1<<level));
   }
   filter = filter_disambiguation (o->filter, std_dev);
