@@ -15,6 +15,7 @@ static const char* op_c_source =
 " *                                                                            \n"
 " * Copyright 2006 Dominik Ernst <dernst@gmx.de>                               \n"
 " * Copyright 2013 Massimo Valentini <mvalentini@src.gnome.org>                \n"
+" *           2017 Øyvind Kolås <pippin@gimp.org>                            \n"
 " *                                                                            \n"
 " * Recursive Gauss IIR Filter as described by Young / van Vliet               \n"
 " * in \"Signal Processing 44 (1995) 139 - 151\"                               \n"
@@ -589,7 +590,6 @@ static const char* op_c_source =
 "filter_disambiguation (GeglGblur1dFilter filter,                              \n"
 "                       gfloat            std_dev)                             \n"
 "{                                                                             \n"
-"  return GEGL_GBLUR_1D_IIR;                                                   \n"
 "  if (filter == GEGL_GBLUR_1D_AUTO)                                           \n"
 "    {                                                                         \n"
 "      /* Threshold 1.0 is arbitrary */                                        \n"
@@ -662,7 +662,7 @@ static const char* op_c_source =
 "                                       const GeglRectangle *output_roi)       \n"
 "{                                                                             \n"
 "  GeglRectangle        required_for_output = { 0, };                          \n"
-"  GeglProperties          *o       = GEGL_PROPERTIES (operation);             \n"
+"  GeglProperties      *o       = GEGL_PROPERTIES (operation);                 \n"
 "  GeglGblur1dFilter    filter  = filter_disambiguation (o->filter, o->std_dev);\n"
 "                                                                              \n"
 "  if (filter == GEGL_GBLUR_1D_IIR)                                            \n"
@@ -733,7 +733,7 @@ static const char* op_c_source =
 "                                 const GeglRectangle *output_roi)             \n"
 "{                                                                             \n"
 "  GeglRectangle      cached_region;                                           \n"
-"  GeglProperties        *o       = GEGL_PROPERTIES (operation);               \n"
+"  GeglProperties    *o       = GEGL_PROPERTIES (operation);                   \n"
 "  GeglGblur1dFilter  filter  = filter_disambiguation (o->filter, o->std_dev); \n"
 "                                                                              \n"
 "  cached_region = *output_roi;                                                \n"
@@ -804,8 +804,6 @@ static const char* op_c_source =
 "  GeglAbyssPolicy   abyss_policy = to_gegl_policy (o->abyss_policy);          \n"
 "                                                                              \n"
 "  GeglRectangle rect2;                                                        \n"
-"  //level = 0; // mipmap rendering doesn't work yet, unsure if it is          \n"
-"             // the code in the op or in GeglBuffer                           \n"
 "  if (level)                                                                  \n"
 "  {                                                                           \n"
 "    rect2 = *result;                                                          \n"
@@ -813,7 +811,7 @@ static const char* op_c_source =
 "    rect2.y      = result->y >> level;                                        \n"
 "    rect2.width  = result->width >> level;                                    \n"
 "    rect2.height = result->height >> level;                                   \n"
-"   // result = &rect2;                                                        \n"
+"    result = &rect2;                                                          \n"
 "    std_dev = std_dev * (1.0/(1<<level));                                     \n"
 "  }                                                                           \n"
 "  filter = filter_disambiguation (o->filter, std_dev);                        \n"
