@@ -52,7 +52,7 @@ static const char* op_c_source =
 "         const GeglRectangle *roi,                                            \n"
 "         gint                 level)                                          \n"
 "{                                                                             \n"
-"  memcpy (out_buf, in_buf, sizeof (gfloat) * 2 * samples);                    \n"
+"  memmove (out_buf, in_buf, sizeof (gfloat) * 2 * samples);                   \n"
 "  return TRUE;                                                                \n"
 "}                                                                             \n"
 "                                                                              \n"
