@@ -44,28 +44,31 @@ static const char* op_c_source =
 "static void                                                                   \n"
 "attach (GeglOperation *operation)                                             \n"
 "{                                                                             \n"
-"  GeglNode *gegl, *input, *output, *invert, *blur, *opacity, *over, *contrast;\n"
+"  GeglNode *gegl, *input, *output, *invert, *blur, *opacity, *over, *sub, *mul, *add, *gam0, *gam1;\n"
 "                                                                              \n"
 "  gegl = operation->node;                                                     \n"
 "  input = gegl_node_get_input_proxy (gegl, \"input\");                        \n"
 "  output = gegl_node_get_output_proxy (gegl, \"output\");                     \n"
-"  invert = gegl_node_new_child (gegl, \"operation\", \"gegl:invert-linear\", NULL);\n"
+"  invert = gegl_node_new_child (gegl, \"operation\", \"gegl:invert-gamma\", NULL);\n"
 "  blur = gegl_node_new_child (gegl, \"operation\", \"gegl:gaussian-blur\", NULL);\n"
 "  opacity = gegl_node_new_child (gegl, \"operation\", \"gegl:opacity\",       \n"
 "                                 \"value\", 0.5, NULL);                       \n"
-"  over = gegl_node_new_child (gegl, \"operation\", \"gegl:over\", NULL);      \n"
-"  contrast = gegl_node_new_child (gegl, \"operation\",                        \n"
-"                                  \"gegl:brightness-contrast\", NULL);        \n"
+"  over = gegl_node_new_child (gegl, \"operation\", \"gegl:over\", \"srgb\", TRUE, NULL);\n"
+"  gam0 = gegl_node_new_child (gegl, \"operation\", \"gegl:gamma\", \"value\", 0.4545, NULL);\n"
+"  sub = gegl_node_new_child (gegl, \"operation\", \"gegl:add\", \"value\", -0.5, NULL);\n"
+"  mul = gegl_node_new_child (gegl, \"operation\", \"gegl:multiply\", \"value\", 1.0, NULL);\n"
+"  add = gegl_node_new_child (gegl, \"operation\", \"gegl:add\", \"value\", 0.5, NULL);\n"
+"  gam1 = gegl_node_new_child (gegl, \"operation\", \"gegl:gamma\", \"value\", 2.2, NULL);\n"
 "                                                                              \n"
-"  gegl_node_link_many (input, invert, blur, opacity, NULL);                   \n"
+"  gegl_node_link_many (input, blur, invert, opacity, NULL);                   \n"
 "  gegl_node_connect_to (opacity, \"output\", over, \"aux\");                  \n"
-"  gegl_node_link_many (input, over, contrast, output, NULL);                  \n"
+"  gegl_node_link_many (input, over, gam0, sub, mul, add, gam1, output, NULL); \n"
 "                                                                              \n"
 "  gegl_operation_meta_redirect (operation, \"std-dev\", blur, \"std-dev-x\"); \n"
 "  gegl_operation_meta_redirect (operation, \"std-dev\", blur, \"std-dev-y\"); \n"
-"  gegl_operation_meta_redirect (operation, \"contrast\", contrast, \"contrast\");\n"
+"  gegl_operation_meta_redirect (operation, \"contrast\", mul, \"value\");     \n"
 "                                                                              \n"
-"  gegl_operation_meta_watch_nodes (operation, invert, blur, opacity, over, contrast, NULL);\n"
+"  gegl_operation_meta_watch_nodes (operation, blur, mul, NULL);               \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static void                                                                   \n"
