@@ -34,10 +34,6 @@ static const char* op_c_source =
 "property_int (n, _(\"Iterations\"), 3)                                        \n"
 "  value_range     (0, 20)                                                     \n"
 "                                                                              \n"
-"  /* XXX: what? */                                                            \n"
-"property_double (seed, _(\"Random seed\"), 0.0)                               \n"
-"  ui_range (0.0, 100.0)                                                       \n"
-"                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
 "#define GEGL_OP_POINT_RENDER                                                  \n"
