@@ -408,6 +408,32 @@ static const char* op_c_source =
 "  return *in_rect;                                                            \n"
 "}                                                                             \n"
 "                                                                              \n"
+"/* Pass-through when iterations parameter is set to zero */                   \n"
+"                                                                              \n"
+"static gboolean                                                               \n"
+"operation_process (GeglOperation        *operation,                           \n"
+"                   GeglOperationContext *context,                             \n"
+"                   const gchar          *output_prop,                         \n"
+"                   const GeglRectangle  *result,                              \n"
+"                   gint                  level)                               \n"
+"{                                                                             \n"
+"  GeglOperationClass  *operation_class;                                       \n"
+"  GeglProperties      *o = GEGL_PROPERTIES (operation);                       \n"
+"                                                                              \n"
+"  operation_class = GEGL_OPERATION_CLASS (gegl_op_parent_class);              \n"
+"                                                                              \n"
+"  if (! o->iterations)                                                        \n"
+"    {                                                                         \n"
+"      gpointer in = gegl_operation_context_get_object (context, \"input\");   \n"
+"      gegl_operation_context_take_object (context, \"output\",                \n"
+"                                          g_object_ref (G_OBJECT (in)));      \n"
+"      return TRUE;                                                            \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  return operation_class->process (operation, context, output_prop, result,   \n"
+"                                   gegl_operation_context_get_level (context));\n"
+"}                                                                             \n"
+"                                                                              \n"
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
@@ -417,8 +443,9 @@ static const char* op_c_source =
 "  operation_class  = GEGL_OPERATION_CLASS (klass);                            \n"
 "  filter_class     = GEGL_OPERATION_FILTER_CLASS (klass);                     \n"
 "                                                                              \n"
-"  filter_class->process   = process;                                          \n"
-"  operation_class->prepare = prepare;                                         \n"
+"  filter_class->process           = process;                                  \n"
+"  operation_class->process        = operation_process;                        \n"
+"  operation_class->prepare        = prepare;                                  \n"
 "  operation_class->opencl_support = TRUE;                                     \n"
 "                                                                              \n"
 "  operation_class->get_bounding_box = get_bounding_box;                       \n"
