@@ -18,7 +18,7 @@ static const char* op_c_source =
 " * Copyright 1996 Torsten Martinsen                                           \n"
 " * Copyright 2000 Tim Copperfield <timecop@japan.co.jp>                       \n"
 " * Copyright 2012 Maxime Nicco <maxime.nicco@gmail.com>                       \n"
-" * Copyright 2012 Øyvind Kolås <pippin@gimp.org>                            \n"
+" * Copyright 2012,2016 Øyvind Kolås <pippin@gimp.org>                       \n"
 " */                                                                           \n"
 "                                                                              \n"
 "#include \"config.h\"                                                         \n"
@@ -29,6 +29,13 @@ static const char* op_c_source =
 "property_boolean (correlated, _(\"Correlated noise\"), FALSE)                 \n"
 "                                                                              \n"
 "property_boolean (independent, _(\"Independent RGB\"), TRUE)                  \n"
+"   description (_(\"Control amount of noise for each RGB channel separately\"))\n"
+"                                                                              \n"
+"property_boolean (linear, _(\"Linear RGB\"), TRUE)                            \n"
+"   description (_(\"Operate on linearized RGB color data\"))                  \n"
+"                                                                              \n"
+"property_boolean (gaussian, _(\"Gaussian distribution\"), TRUE)               \n"
+"   description (_(\"Use a gaussian noise distribution, when unticked a linear noise distribution is used instead\"))\n"
 "                                                                              \n"
 "property_double (red, _(\"Red\"), 0.20)                                       \n"
 "   value_range  (0.0, 1.0)                                                    \n"
@@ -63,10 +70,10 @@ static const char* op_c_source =
 " * Ratio method (Kinderman-Monahan); see Knuth v2, 3rd ed, p130               \n"
 " * K+M, ACM Trans Math Software 3 (1977) 257-260.                             \n"
 "*/                                                                            \n"
-"static gdouble                                                                \n"
-"gauss (GeglRandom *rand, int *i, int xx, int yy)                              \n"
+"static gfloat                                                                 \n"
+"noise_gauss (GeglRandom *rand, int *i, int xx, int yy)                        \n"
 "{                                                                             \n"
-"  gdouble u, v, x;                                                            \n"
+"  gfloat u, v, x;                                                             \n"
 "                                                                              \n"
 "  do                                                                          \n"
 "  {                                                                           \n"
@@ -77,18 +84,33 @@ static const char* op_c_source =
 "    while (u == 0);                                                           \n"
 "                                                                              \n"
 "    /* Const 1.715... = sqrt(8/e) */                                          \n"
-"    x = 1.71552776992141359295 * (v - 0.5) / u;                               \n"
+"    x = 1.71552776992141359295 * (v - 0.5f) / u;                              \n"
 "  }                                                                           \n"
-"  while (x * x > -4.0 * log (u));                                             \n"
+"  while (x * x > -4.0f * logf (u));                                           \n"
 "                                                                              \n"
 "  return x;                                                                   \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static gfloat                                                                 \n"
+"noise_linear (GeglRandom *rand, int *i, int xx, int yy)                       \n"
+"{                                                                             \n"
+"  return gegl_random_float (rand, xx, yy, 0, (*i)++) * 2 - 1.0;               \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static void                                                                   \n"
 "prepare (GeglOperation *operation)                                            \n"
 "{                                                                             \n"
-"  gegl_operation_set_format (operation, \"input\", babl_format (\"RGBA float\"));\n"
-"  gegl_operation_set_format (operation, \"output\", babl_format (\"RGBA float\"));\n"
+"  GeglProperties *o  = GEGL_PROPERTIES (operation);                           \n"
+"  if (o->linear)                                                              \n"
+"    {                                                                         \n"
+"      gegl_operation_set_format (operation, \"input\", babl_format (\"RGBA float\"));\n"
+"      gegl_operation_set_format (operation, \"output\", babl_format (\"RGBA float\"));\n"
+"    }                                                                         \n"
+"  else                                                                        \n"
+"    {                                                                         \n"
+"      gegl_operation_set_format (operation, \"input\", babl_format (\"R'G'B'A float\"));\n"
+"      gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B'A float\"));\n"
+"    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static gboolean                                                               \n"
@@ -109,6 +131,7 @@ static const char* op_c_source =
 "  gfloat   tmp;                                                               \n"
 "  gfloat   * GEGL_ALIGNED in_pixel;                                           \n"
 "  gfloat   * GEGL_ALIGNED out_pixel;                                          \n"
+"  gfloat   (*noise_fun) (GeglRandom *rand, int *i, int xx, int yy) = noise_gauss;\n"
 "                                                                              \n"
 "  in_pixel   = in_buf;                                                        \n"
 "  out_pixel  = out_buf;                                                       \n"
@@ -118,6 +141,9 @@ static const char* op_c_source =
 "  noise[2] = o->blue;                                                         \n"
 "  noise[3] = o->alpha;                                                        \n"
 "                                                                              \n"
+"  if (o->gaussian == FALSE)                                                   \n"
+"    noise_fun = noise_linear;                                                 \n"
+"                                                                              \n"
 "  x = roi->x;                                                                 \n"
 "  y = roi->y;                                                                 \n"
 "                                                                              \n"
@@ -126,7 +152,7 @@ static const char* op_c_source =
 "    for (b = 0; b < 4; b++)                                                   \n"
 "    {                                                                         \n"
 "      if (b == 0 || o->independent || b == 3 )                                \n"
-"         noise_coeff = noise[b] * gauss (o->rand, &rint, x, y) * 0.5;         \n"
+"         noise_coeff = noise[b] * noise_fun (o->rand, &rint, x, y) * 0.5;     \n"
 "                                                                              \n"
 "      if (noise[b] > 0.0)                                                     \n"
 "      {                                                                       \n"
