@@ -249,11 +249,15 @@ static const char* op_c_source =
 "  for (j = roi->y; j < roi->y + roi->height; j++)                             \n"
 "    {                                                                         \n"
 "      y = ((gdouble) j) / o->height;                                          \n"
+"      if (level)                                                              \n"
+"        y *= (1<<level);                                                      \n"
 "      for (i = roi->x; i < roi->x + roi->width; i++)                          \n"
 "        {                                                                     \n"
 "          gdouble c;                                                          \n"
 "                                                                              \n"
 "          x = ((gdouble) i) / o->width;                                       \n"
+"          if (level)                                                          \n"
+"            x *= (1<<level);                                                  \n"
 "                                                                              \n"
 "          c = 0.5 * sin(p->c31 * x + p->c32 * y + p->c33);                    \n"
 "                                                                              \n"
