@@ -46,10 +46,12 @@ static const char* op_c_source =
 "                                                                              \n"
 "static void                                                                   \n"
 "snn_mean (GeglBuffer          *src,                                           \n"
+"          const GeglRectangle *src_rect,                                      \n"
 "          GeglBuffer          *dst,                                           \n"
 "          const GeglRectangle *dst_rect,                                      \n"
 "          gdouble              radius,                                        \n"
-"          gint                 pairs);                                        \n"
+"          gint                 pairs,                                         \n"
+"          gint                 level);                                        \n"
 "                                                                              \n"
 "                                                                              \n"
 "static void prepare (GeglOperation *operation)                                \n"
@@ -76,7 +78,6 @@ static const char* op_c_source =
 "         gint                 level)                                          \n"
 "{                                                                             \n"
 "  GeglProperties      *o = GEGL_PROPERTIES (operation);                       \n"
-"  GeglBuffer          *temp_in;                                               \n"
 "  GeglRectangle        compute;                                               \n"
 "                                                                              \n"
 "  if (gegl_operation_use_opencl (operation))                                  \n"
@@ -91,10 +92,7 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "  else                                                                        \n"
 "    {                                                                         \n"
-"      temp_in = gegl_buffer_create_sub_buffer (input, &compute);              \n"
-"                                                                              \n"
-"      snn_mean (temp_in, output, result, o->radius, o->pairs);                \n"
-"      g_object_unref (temp_in);                                               \n"
+"      snn_mean (input, &compute, output, result, o->radius, o->pairs, level); \n"
 "    }                                                                         \n"
 "                                                                              \n"
 "  return  TRUE;                                                               \n"
@@ -113,23 +111,42 @@ static const char* op_c_source =
 "                                                                              \n"
 "static void                                                                   \n"
 "snn_mean (GeglBuffer          *src,                                           \n"
+"          const GeglRectangle *src_rect,                                      \n"
 "          GeglBuffer          *dst,                                           \n"
 "          const GeglRectangle *dst_rect,                                      \n"
 "          gdouble              dradius,                                       \n"
-"          gint                 pairs)                                         \n"
+"          gint                 pairs,                                         \n"
+"          gint                 level)                                         \n"
 "{                                                                             \n"
 "  gint x,y;                                                                   \n"
 "  gint offset;                                                                \n"
 "  gfloat *src_buf;                                                            \n"
 "  gfloat *dst_buf;                                                            \n"
 "  gint radius = dradius;                                                      \n"
-"  gint src_width = gegl_buffer_get_width (src);                               \n"
-"  gint src_height = gegl_buffer_get_height (src);                             \n"
+"  GeglRectangle src_rect_scaled, dst_rect_scaled;                             \n"
+"  if (level)                                                                  \n"
+"  {                                                                           \n"
+"    src_rect_scaled = *src_rect;                                              \n"
+"    dst_rect_scaled = *dst_rect;                                              \n"
+"    src_rect_scaled.x >>= level;                                              \n"
+"    src_rect_scaled.y >>= level;                                              \n"
+"    src_rect_scaled.width >>= level;                                          \n"
+"    src_rect_scaled.height >>= level;                                         \n"
+"    dst_rect_scaled.x >>= level;                                              \n"
+"    dst_rect_scaled.y >>= level;                                              \n"
+"    dst_rect_scaled.width >>= level;                                          \n"
+"    dst_rect_scaled.height >>= level;                                         \n"
+"    src_rect = &src_rect_scaled;                                              \n"
+"    dst_rect = &dst_rect_scaled;                                              \n"
+"    dradius /= (1<<level);                                                    \n"
+"  }                                                                           \n"
+"  radius = dradius;                                                           \n"
 "                                                                              \n"
-"  src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);           \n"
+"                                                                              \n"
+"  src_buf = g_new0 (gfloat, src_rect->width * src_rect->height * 4);          \n"
 "  dst_buf = g_new0 (gfloat, dst_rect->width * dst_rect->height * 4);          \n"
 "                                                                              \n"
-"  gegl_buffer_get (src, NULL, 1.0, babl_format (\"RGBA float\"), src_buf,     \n"
+"  gegl_buffer_get (src, src_rect, 1.0/(1<<level), babl_format (\"RGBA float\"), src_buf,\n"
 "                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);                     \n"
 "                                                                              \n"
 "  offset = 0;                                                                 \n"
@@ -138,7 +155,7 @@ static const char* op_c_source =
 "    {                                                                         \n"
 "      gfloat *center_pix;                                                     \n"
 "                                                                              \n"
-"      center_pix = src_buf + ((radius) + (y+radius)* src_width)*4;            \n"
+"      center_pix = src_buf + ((radius) + (y+radius)* src_rect->width)*4;      \n"
 "                                                                              \n"
 "      for (x=0; x<dst_rect->width; x++)                                       \n"
 "        {                                                                     \n"
@@ -176,10 +193,10 @@ static const char* op_c_source =
 "                    /* check which member of the symmetric quadruple to use */\n"
 "                    for (i=0;i<pairs*2;i++)                                   \n"
 "                      {                                                       \n"
-"                        if (xs[i] >= 0 && xs[i] < src_width &&                \n"
-"                            ys[i] >= 0 && ys[i] < src_height)                 \n"
+"                        if (xs[i] >= 0 && xs[i] < src_rect->width &&          \n"
+"                            ys[i] >= 0 && ys[i] < src_rect->height)           \n"
 "                          {                                                   \n"
-"                            gfloat *tpix = src_buf + (xs[i]+ys[i]* src_width)*4;\n"
+"                            gfloat *tpix = src_buf + (xs[i]+ys[i]* src_rect->width)*4;\n"
 "                            gfloat diff = colordiff (tpix, center_pix);       \n"
 "                            if (diff < best_diff)                             \n"
 "                              {                                               \n"
@@ -209,7 +226,7 @@ static const char* op_c_source =
 "          center_pix += 4;                                                    \n"
 "        }                                                                     \n"
 "    }                                                                         \n"
-"  gegl_buffer_set (dst, dst_rect, 0, babl_format (\"RGBA float\"), dst_buf,   \n"
+"  gegl_buffer_set (dst, dst_rect, level, babl_format (\"RGBA float\"), dst_buf,\n"
 "                   GEGL_AUTO_ROWSTRIDE);                                      \n"
 "  g_free (src_buf);                                                           \n"
 "  g_free (dst_buf);                                                           \n"
