@@ -59,31 +59,31 @@ static const char* op_c_source =
 "                                                                              \n"
 "  gegl = gegl_node_new ();                                                    \n"
 "  image = gegl_node_new_child (gegl,                                          \n"
-"                \"operation\", \"gegl:buffer-source\",                        \n"
-"                \"buffer\", input,                                            \n"
-"                NULL);                                                        \n"
+"                               \"operation\", \"gegl:buffer-source\",         \n"
+"                               \"buffer\",    input,                          \n"
+"                               NULL);                                         \n"
 "                                                                              \n"
 "  radius   = fabs (glow_radius) + 1.0;                                        \n"
 "  std_dev = sqrt (-(radius * radius) / (2 * log (1.0 / 255.0)));              \n"
 "                                                                              \n"
 "  blur =  gegl_node_new_child (gegl,                                          \n"
-"                \"operation\", \"gegl:gaussian-blur\",                        \n"
-"                \"std_dev_x\", std_dev,                                       \n"
-"                \"std_dev_y\", std_dev,                                       \n"
-"                \"abyss-policy\", \"none\",                                   \n"
-"                NULL);                                                        \n"
+"                               \"operation\",    \"gegl:gaussian-blur\",      \n"
+"                               \"std_dev_x\",    std_dev,                     \n"
+"                               \"std_dev_y\",    std_dev,                     \n"
+"                               \"abyss-policy\", 0,                           \n"
+"                               NULL);                                         \n"
 "                                                                              \n"
 "  crop =  gegl_node_new_child (gegl,                                          \n"
-"                \"operation\", \"gegl:crop\",                                 \n"
-"                \"x\",     (gdouble) result->x,                               \n"
-"                \"y\",     (gdouble) result->y,                               \n"
-"                \"width\", (gdouble) result->width,                           \n"
-"                \"height\",(gdouble) result->height,                          \n"
-"                NULL);                                                        \n"
+"                               \"operation\", \"gegl:crop\",                  \n"
+"                               \"x\",         (gdouble) result->x,            \n"
+"                               \"y\",         (gdouble) result->y,            \n"
+"                               \"width\",     (gdouble) result->width,        \n"
+"                               \"height\",    (gdouble) result->height,       \n"
+"                               NULL);                                         \n"
 "                                                                              \n"
 "  write = gegl_node_new_child (gegl,                                          \n"
-"                \"operation\", \"gegl:buffer-sink\",                          \n"
-"                \"buffer\", &dest, NULL);                                     \n"
+"                               \"operation\", \"gegl:buffer-sink\",           \n"
+"                               \"buffer\", &dest, NULL);                      \n"
 "                                                                              \n"
 "  gegl_node_link_many (image, blur, crop, write, NULL);                       \n"
 "  gegl_node_process (write);                                                  \n"
@@ -129,7 +129,7 @@ static const char* op_c_source =
 "         gint                 level)                                          \n"
 "{                                                                             \n"
 "  GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);     \n"
-"  GeglProperties              *o    = GEGL_PROPERTIES (operation);            \n"
+"  GeglProperties          *o    = GEGL_PROPERTIES (operation);                \n"
 "                                                                              \n"
 "  GeglBuffer *dest, *dest_tmp;                                                \n"
 "                                                                              \n"
