@@ -42,7 +42,7 @@ static const char* op_c_source =
 "} AutostretchData;                                                            \n"
 "                                                                              \n"
 "static void                                                                   \n"
-"buffer_get_auto_strech_data (GeglBuffer      *buffer,                         \n"
+"buffer_get_auto_stretch_data (GeglBuffer      *buffer,                        \n"
 "                             AutostretchData *data)                           \n"
 "{                                                                             \n"
 "  gfloat smin =  G_MAXFLOAT;                                                  \n"
@@ -142,7 +142,7 @@ static const char* op_c_source =
 "  AutostretchData     data;                                                   \n"
 "  GeglBufferIterator *gi;                                                     \n"
 "                                                                              \n"
-"  buffer_get_auto_strech_data (input, &data);                                 \n"
+"  buffer_get_auto_stretch_data (input, &data);                                \n"
 "  clean_autostretch_data (&data);                                             \n"
 "                                                                              \n"
 "  gi = gegl_buffer_iterator_new (input, result, 0, babl_format (\"HSVA float\"),\n"
