@@ -26,7 +26,7 @@ static const char* op_c_source =
 "property_double (value, _(\"Threshold\"), 0.5)                                \n"
 "    value_range (-200, 200)                                                   \n"
 "    ui_range    (-1, 2)                                                       \n"
-"    description(_(\"Scalar threshold level (overriden if an auxiliary input buffer is provided.).\"))\n"
+"    description(_(\"Scalar threshold level (overridden if an auxiliary input buffer is provided.).\"))\n"
 "                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
