@@ -53,6 +53,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "#define GEGL_OP_FILTER                                                        \n"
 "#define GEGL_OP_C_SOURCE warp.c                                               \n"
+"#define GEGL_OP_NAME     warp                                                 \n"
 "                                                                              \n"
 "#include \"gegl-plugin.h\"                                                    \n"
 "#include \"gegl-path.h\"                                                      \n"
@@ -185,7 +186,7 @@ static const char* op_c_source =
 "      calc_lut (o);                                                           \n"
 "    }                                                                         \n"
 "                                                                              \n"
-"  radius = sqrt(x*x+y*y);                                                     \n"
+"  radius = hypot (x, y);                                                      \n"
 "                                                                              \n"
 "  if (radius < 0.5 * o->size + 1)                                             \n"
 "    {                                                                         \n"
@@ -215,14 +216,22 @@ static const char* op_c_source =
 "  WarpPrivate         *priv = (WarpPrivate*) o->user_data;                    \n"
 "  GeglBufferIterator  *it;                                                    \n"
 "  const Babl          *format;                                                \n"
-"  gdouble              influence;                                             \n"
+"  gdouble              stamp_force, influence;                                \n"
 "  gdouble              x_mean = 0.0;                                          \n"
 "  gdouble              y_mean = 0.0;                                          \n"
 "  gint                 x_iter, y_iter;                                        \n"
-"  GeglRectangle        area = {x - o->size / 2.0,                             \n"
-"                               y - o->size / 2.0,                             \n"
-"                               o->size,                                       \n"
-"                               o->size};                                      \n"
+"  GeglRectangle        area;                                                  \n"
+"  const GeglRectangle *src_extent;                                            \n"
+"  gfloat              *srcbuf, *stampbuf;                                     \n"
+"  gint                 buf_rowstride = 0;                                     \n"
+"  gfloat               s = 0, c = 0;                                          \n"
+"                                                                              \n"
+"  area.x = floor (x - o->size / 2.0);                                         \n"
+"  area.y = floor (y - o->size / 2.0);                                         \n"
+"  area.width   = ceil (x + o->size / 2.0);                                    \n"
+"  area.height  = ceil (y + o->size / 2.0);                                    \n"
+"  area.width  -= area.x;                                                      \n"
+"  area.height -= area.y;                                                      \n"
 "                                                                              \n"
 "  /* first point of the stroke */                                             \n"
 "  if (!priv->last_point_set)                                                  \n"
@@ -263,69 +272,131 @@ static const char* op_c_source =
 "      x_mean /= pixel_count;                                                  \n"
 "      y_mean /= pixel_count;                                                  \n"
 "    }                                                                         \n"
-"                                                                              \n"
-"  it = gegl_buffer_iterator_new (priv->buffer, &area, 0, format,              \n"
-"                                 GEGL_ACCESS_READWRITE, GEGL_ABYSS_NONE);     \n"
-"                                                                              \n"
-"  while (gegl_buffer_iterator_next (it))                                      \n"
+"  else if (o->behavior == GEGL_WARP_BEHAVIOR_SWIRL_CW ||                      \n"
+"           o->behavior == GEGL_WARP_BEHAVIOR_SWIRL_CCW)                       \n"
 "    {                                                                         \n"
-"      /* iterate inside the stamp roi */                                      \n"
-"      gint    n_pixels = it->length;                                          \n"
-"      gfloat *coords   = it->data[0];                                         \n"
+"      /* swirl by 5 degrees per stamp (for strength 100).                     \n"
+"       * not exactly sin/cos factors,                                         \n"
+"       * since we calculate an off-center offset-vector */                    \n"
 "                                                                              \n"
-"      x_iter = it->roi->x; /* initial x         */                            \n"
-"      y_iter = it->roi->y; /* and y coordinates */                            \n"
+"      /* note that this is fudged for stamp_force < 1.0 and                   \n"
+"       * results in a slight upscaling there. It is a compromise              \n"
+"       * between exactness and calculation speed. */                          \n"
+"      s = sin (0.01 * o->strength * 5.0 / 180 * G_PI);                        \n"
+"      c = cos (0.01 * o->strength * 5.0 / 180 * G_PI) - 1.0;                  \n"
+"    }                                                                         \n"
 "                                                                              \n"
-"      while (n_pixels--)                                                      \n"
+"  srcbuf = gegl_buffer_linear_open (priv->buffer, NULL, &buf_rowstride, NULL);\n"
+"  buf_rowstride /= sizeof (gfloat);                                           \n"
+"  src_extent = gegl_buffer_get_extent (priv->buffer);                         \n"
+"                                                                              \n"
+"  stampbuf = g_new0 (gfloat, 2 * area.height * area.width);                   \n"
+"                                                                              \n"
+"  for (y_iter = 0; y_iter < area.height; y_iter++)                            \n"
+"    {                                                                         \n"
+"      for (x_iter = 0; x_iter < area.width; x_iter++)                         \n"
 "        {                                                                     \n"
-"          influence = 0.01 * o->strength * get_stamp_force (o,                \n"
-"                                                            x_iter - x,       \n"
-"                                                            y_iter - y);      \n"
+"          gfloat nvx, nvy;                                                    \n"
+"          gfloat xi, yi;                                                      \n"
+"          gfloat *vals;                                                       \n"
+"          gint dx, dy;                                                        \n"
+"          gfloat weight_x, weight_y;                                          \n"
+"          gfloat *srcptr;                                                     \n"
+"                                                                              \n"
+"          xi = area.x + x_iter;                                               \n"
+"          xi += -x + 0.5;                                                     \n"
+"          yi = area.y + y_iter;                                               \n"
+"          yi += -y + 0.5;                                                     \n"
+"                                                                              \n"
+"          stamp_force = get_stamp_force (o, xi, yi);                          \n"
+"          influence = 0.01 * o->strength * stamp_force;                       \n"
 "                                                                              \n"
 "          switch (o->behavior)                                                \n"
 "            {                                                                 \n"
 "              case GEGL_WARP_BEHAVIOR_MOVE:                                   \n"
-"                coords[0] += influence * (priv->last_x - x);                  \n"
-"                coords[1] += influence * (priv->last_y - y);                  \n"
+"                nvx = influence * (priv->last_x - x);                         \n"
+"                nvy = influence * (priv->last_y - y);                         \n"
 "                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_GROW:                                   \n"
-"                coords[0] -= influence * 0.1 * (x_iter - x);                  \n"
-"                coords[1] -= influence * 0.1 * (y_iter - y);                  \n"
+"                nvx = influence * -0.1 * xi;                                  \n"
+"                nvy = influence * -0.1 * yi;                                  \n"
 "                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_SHRINK:                                 \n"
-"                coords[0] += influence * 0.1 * (x_iter - x);                  \n"
-"                coords[1] += influence * 0.1 * (y_iter - y);                  \n"
+"                nvx = influence * 0.1 * xi;                                   \n"
+"                nvy = influence * 0.1 * yi;                                   \n"
 "                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_SWIRL_CW:                               \n"
-"                coords[0] += 3.0 * influence * 0.1 * (y_iter - y);            \n"
-"                coords[1] -= 5.0 * influence * 0.1 * (x_iter - x);            \n"
+"                nvx = stamp_force * ( c * xi + s * yi);                       \n"
+"                nvy = stamp_force * (-s * xi + c * yi);                       \n"
 "                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_SWIRL_CCW:                              \n"
-"                coords[0] -= 3.0 * influence * 0.1 * (y_iter - y);            \n"
-"                coords[1] += 5.0 * influence * 0.1 * (x_iter - x);            \n"
+"                nvx = stamp_force * ( c * xi - s * yi);                       \n"
+"                nvy = stamp_force * ( s * xi + c * yi);                       \n"
 "                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_ERASE:                                  \n"
-"                coords[0] *= 1.0 - MIN (influence, 1.0);                      \n"
-"                coords[1] *= 1.0 - MIN (influence, 1.0);                      \n"
-"                break;                                                        \n"
 "              case GEGL_WARP_BEHAVIOR_SMOOTH:                                 \n"
-"                coords[0] -= influence * (coords[0] - x_mean);                \n"
-"                coords[1] -= influence * (coords[1] - y_mean);                \n"
+"              default:                                                        \n"
+"                nvx = 0.0;                                                    \n"
+"                nvy = 0.0;                                                    \n"
 "                break;                                                        \n"
 "            }                                                                 \n"
 "                                                                              \n"
-"          coords += 2;                                                        \n"
+"          vals = stampbuf + (y_iter * area.width + x_iter) * 2;               \n"
 "                                                                              \n"
-"          /* update x and y coordinates */                                    \n"
-"          x_iter++;                                                           \n"
-"          if (x_iter >= (it->roi->x + it->roi->width))                        \n"
+"          dx = floorf (nvx);                                                  \n"
+"          dy = floorf (nvy);                                                  \n"
+"                                                                              \n"
+"          if (area.x + x_iter + dx     <  src_extent->x                     ||\n"
+"              area.x + x_iter + dx + 1 >= src_extent->x + src_extent->width ||\n"
+"              area.y + y_iter + dy     <  src_extent->y                     ||\n"
+"              area.y + y_iter + dy + 1 >= src_extent->y + src_extent->height) \n"
 "            {                                                                 \n"
-"              x_iter = it->roi->x;                                            \n"
-"              y_iter++;                                                       \n"
+"              continue;                                                       \n"
+"            }                                                                 \n"
+"                                                                              \n"
+"          srcptr = srcbuf + (area.y - src_extent->y + y_iter + dy) * buf_rowstride +\n"
+"                            (area.x - src_extent->x + x_iter + dx) * 2;       \n"
+"                                                                              \n"
+"          if (o->behavior == GEGL_WARP_BEHAVIOR_ERASE)                        \n"
+"            {                                                                 \n"
+"              vals[0] = srcptr[0] * (1.0 - MIN (influence, 1.0));             \n"
+"              vals[1] = srcptr[1] * (1.0 - MIN (influence, 1.0));             \n"
+"            }                                                                 \n"
+"          else if (o->behavior == GEGL_WARP_BEHAVIOR_SMOOTH)                  \n"
+"            {                                                                 \n"
+"              vals[0] = (1.0 - influence) * srcptr[0] + influence * x_mean;   \n"
+"              vals[1] = (1.0 - influence) * srcptr[1] + influence * y_mean;   \n"
+"            }                                                                 \n"
+"          else                                                                \n"
+"            {                                                                 \n"
+"              weight_x = nvx - dx;                                            \n"
+"              weight_y = nvy - dy;                                            \n"
+"                                                                              \n"
+"              /* bilinear interpolation of the vectors */                     \n"
+"                                                                              \n"
+"              vals[0]  = srcptr[0] * (1.0 - weight_x) * (1.0 - weight_y);     \n"
+"              vals[1]  = srcptr[1] * (1.0 - weight_x) * (1.0 - weight_y);     \n"
+"                                                                              \n"
+"              vals[0] += srcptr[2] * weight_x * (1.0 - weight_y);             \n"
+"              vals[1] += srcptr[3] * weight_x * (1.0 - weight_y);             \n"
+"                                                                              \n"
+"              vals[0] += srcptr[buf_rowstride + 0] * (1.0 - weight_x) * weight_y;\n"
+"              vals[1] += srcptr[buf_rowstride + 1] * (1.0 - weight_x) * weight_y;\n"
+"                                                                              \n"
+"              vals[0] += srcptr[buf_rowstride + 2] * weight_x * weight_y;     \n"
+"              vals[1] += srcptr[buf_rowstride + 3] * weight_x * weight_y;     \n"
+"                                                                              \n"
+"              vals[0] += nvx;                                                 \n"
+"              vals[1] += nvy;                                                 \n"
 "            }                                                                 \n"
 "        }                                                                     \n"
 "    }                                                                         \n"
 "                                                                              \n"
+"  gegl_buffer_linear_close (priv->buffer, srcbuf);                            \n"
+"  gegl_buffer_set (priv->buffer, &area, 0, format,                            \n"
+"                   stampbuf, GEGL_AUTO_ROWSTRIDE);                            \n"
+"  g_free (stampbuf);                                                          \n"
+"                                                                              \n"
 "  /* Memorize the stamp location for movement dependant behavior like move */ \n"
 "  priv->last_x = x;                                                           \n"
 "  priv->last_y = y;                                                           \n"
