@@ -58,12 +58,8 @@ static const char* op_c_source =
 "                                                                              \n"
 "  if (p != NULL)                                                              \n"
 "    {                                                                         \n"
-"      if (p->image != NULL)                                                   \n"
-"        jas_image_destroy(p->image);                                          \n"
-"      p->image = NULL;                                                        \n"
-"                                                                              \n"
-"      if (p->file != NULL)                                                    \n"
-"        g_clear_object(&p->file);                                             \n"
+"      g_clear_pointer (&p->image, (GDestroyNotify) jas_image_destroy);        \n"
+"      g_clear_object (&p->file);                                              \n"
 "                                                                              \n"
 "      p->width = p->height = 0;                                               \n"
 "      p->format = NULL;                                                       \n"
