@@ -55,7 +55,9 @@ static const char* op_c_source =
 "      GeglRectangle *rect = gegl_operation_source_get_bounding_box (operation, \"input\");\n"
 "      gchar *name;                                                            \n"
 "      gboolean has_alpha;                                                     \n"
+"      gint bpp;                                                               \n"
 "      gint bps;                                                               \n"
+"      gint stride;                                                            \n"
 "                                                                              \n"
 "      g_object_get (input, \"format\", &format, NULL);                        \n"
 "                                                                              \n"
@@ -68,8 +70,11 @@ static const char* op_c_source =
 "                  bps);                                                       \n"
 "      babl = babl_format (name);                                              \n"
 "                                                                              \n"
-"      temp = g_malloc (rect->width * rect->height * bps);                     \n"
-"      gegl_buffer_get (input, rect, 1.0, babl, temp, GEGL_AUTO_ROWSTRIDE,     \n"
+"      bpp = babl_format_get_bytes_per_pixel (babl);                           \n"
+"      stride = bpp * rect->width;                                             \n"
+"                                                                              \n"
+"      temp = g_malloc0_n (stride, rect->height);                              \n"
+"      gegl_buffer_get (input, rect, 1.0, babl, temp, stride,                  \n"
 "                       GEGL_ABYSS_NONE);                                      \n"
 "      if (temp) {                                                             \n"
 "    *pixbuf = gdk_pixbuf_new_from_data (temp,                                 \n"
@@ -77,7 +82,7 @@ static const char* op_c_source =
 "                        has_alpha,                                            \n"
 "                        bps,                                                  \n"
 "                        rect->width, rect->height,                            \n"
-"                        rect->width * (has_alpha ? 4 : 3) * bps/8,            \n"
+"                        stride,                                               \n"
 "                        (GdkPixbufDestroyNotify) g_free, NULL);               \n"
 "      }                                                                       \n"
 "      else {                                                                  \n"
