@@ -62,12 +62,11 @@ static const char* op_c_source =
 "        TIFFClose(p->tiff);                                                   \n"
 "      else if (p->stream != NULL)                                             \n"
 "        g_output_stream_close(G_OUTPUT_STREAM(p->stream), NULL, NULL);        \n"
-"      if (p->stream != NULL)                                                  \n"
-"        g_clear_object(&p->stream);                                           \n"
+"                                                                              \n"
+"      g_clear_object (&p->stream);                                            \n"
 "      p->tiff = NULL;                                                         \n"
 "                                                                              \n"
-"      if (p->file != NULL)                                                    \n"
-"        g_clear_object(&p->file);                                             \n"
+"      g_clear_object (&p->file);                                              \n"
 "    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
