@@ -61,22 +61,18 @@ cleanup(GeglOperation *operation)
 
   if (p != NULL)
     {
-      if (p->decoder != NULL)
-        WebPIDelete (p->decoder);
-      p->decoder = NULL;
+      g_clear_pointer (&p->decoder, (GDestroyNotify) WebPIDelete);
+
       if (p->config != NULL)
         WebPFreeDecBuffer (&p->config->output);
-      if (p->config != NULL)
-        g_free (p->config);
-      p->config = NULL;
+
+      g_clear_pointer (&p->config, g_free);
 
       if (p->stream != NULL)
         g_input_stream_close (G_INPUT_STREAM (p->stream), NULL, NULL);
-      if (p->stream != NULL)
-        g_clear_object (&p->stream);
 
-      if (p->file != NULL)
-        g_clear_object (&p->file);
+      g_clear_object (&p->stream);
+      g_clear_object (&p->file);
 
       p->width = p->height = 0;
       p->format = NULL;
