@@ -62,22 +62,18 @@ static const char* op_c_source =
 "                                                                              \n"
 "  if (p != NULL)                                                              \n"
 "    {                                                                         \n"
-"      if (p->decoder != NULL)                                                 \n"
-"        WebPIDelete (p->decoder);                                             \n"
-"      p->decoder = NULL;                                                      \n"
+"      g_clear_pointer (&p->decoder, (GDestroyNotify) WebPIDelete);            \n"
+"                                                                              \n"
 "      if (p->config != NULL)                                                  \n"
 "        WebPFreeDecBuffer (&p->config->output);                               \n"
-"      if (p->config != NULL)                                                  \n"
-"        g_free (p->config);                                                   \n"
-"      p->config = NULL;                                                       \n"
+"                                                                              \n"
+"      g_clear_pointer (&p->config, g_free);                                   \n"
 "                                                                              \n"
 "      if (p->stream != NULL)                                                  \n"
 "        g_input_stream_close (G_INPUT_STREAM (p->stream), NULL, NULL);        \n"
-"      if (p->stream != NULL)                                                  \n"
-"        g_clear_object (&p->stream);                                          \n"
 "                                                                              \n"
-"      if (p->file != NULL)                                                    \n"
-"        g_clear_object (&p->file);                                            \n"
+"      g_clear_object (&p->stream);                                            \n"
+"      g_clear_object (&p->file);                                              \n"
 "                                                                              \n"
 "      p->width = p->height = 0;                                               \n"
 "      p->format = NULL;                                                       \n"
